/**
 * This class contains some wrappers functions
 *  to facebook API functions, etc.
 *
 * @author Matanya
 */
var FacebookLib = (function ()
{
	var me = {};

	// The default amount of item we want to take in every "get"
	var POSTS_TAKE_COUNT = 20;
	var COMMENTS_TAKE_COUNT = 50;

	var _appAccessToken;

	var _userAccessToken = "";

	var _userId = "";

	// The facebook appId. we use temporary the default:
	var _appId = "";
	if (SERVICEMAP_URL === DEV_SERVICEMAP_URL)
	{
		_appId = "244523672272758"; // Dev FB app
	}
	else if (SERVICEMAP_URL === QA_SERVICEMAP_URL)
	{
		_appId = "2205671696137617"; // QASite FB app
	} else // = if (SERVICEMAP_URL === PROD_SERVICEMAP_URL) 
	{
		_appId = "209845035304"; // Production FB app
	}
	if (PLATFORM === platformEnum.nativeApp && DEVICE === deviceTypeEnum.winPhone)
	{
		_appId = "209845035304"; // Always use production FB app for WinPhone native
	}

	/**
	* set the _userAccessToken, _userId vars.
	* Call this function only once...
	* NOTE: We can do the "get" functions with the _appAccessToken,
	*  but it is better to use _userAccessToken.
	*
	* @param {string} accessToken - the access token of the user.
	* @param {string} userId - the facebook user id.
	* @author Matanya
	*/
	me.setUser = function (accessToken, userId)
	{
		_userAccessToken = accessToken;
		_userId = userId;
	};

	/**
	* set the _appId var.
	* Call this function only once...
	* @param {string} appId - the facebook app id.
	* @author Matanya
	*/
	me.setAppId = function (appId)
	{
		_appId = appId;
	};

	/**
	* This function convert "facebook time" (=string) to unix time (=integer)
	* 
	* @param {string} facebookDateTime, e.g.: "2011-05-30T14:34:56+0000"
	* @return {integer} dateTime - # of mSec from 1/1/1970
	* @author Matanya
	*/
	//TODO: delete all other places that we use this func in hte code!!!
	me.facebookToUnixTime = function (facebookDateTime)
	{
		facebookDateTime = facebookDateTime.replace('+0000', '');
		var tSplit = facebookDateTime.split('T');

		var dateSplit = tSplit[0].split('-');
		var timeSplit = tSplit[1].split(':');

		var dateTime = new Date(+dateSplit[0], +dateSplit[1] - 1, +dateSplit[2], +timeSplit[0] + 3, +timeSplit[1], +timeSplit[2]);
		return dateTime.getTime();
	};

	/**
	* Get the profile picture of a user by its ID using the access token.
	* 
	* @param userId - the user's ID.
	* @returns the image URL.
	*/
	me.getPictureFromId = function (userId)
	{
		var accessToken = _userAccessToken;
		if (!accessToken)
		{
			accessToken = _appAccessToken;
		}

		// if we have an access token, use it to get the profile picture.
		if (accessToken)
		{
			return 'https://graph.facebook.com/' + userId + '/picture?access_token=' + accessToken;
		}
		else
		{
			return 'http://graph.facebook.com/' + userId + '/picture';
		}

	};

	/**
	* get the _userAccessToken var.
	* NOTE: (according to the "option") If we don't have the _userAccessToken,
	* 	we will "login" to get it, and return it.
	*
	* @author Matanya
	*/
	var getAccessToken = function (cbSuccess, cbFail, options)
	{
		options = options || {};

		function getAccessTokenUsingLogin()
		{
			if (conduitApi.social.isSupported())
			{
				var options = { useLoginForm: true };
				var cbGetAccessSuccess = function (tokens)
				{
					// Update local vars:
					_userAccessToken = tokens.accessToken;
					_userId = tokens.fbId;

					// Return the wanted var:
					Utils.Helpers.funcWrapper(cbSuccess)(_userAccessToken);

				};
				conduitApi.social.getAccessTokens(cbGetAccessSuccess,
												  cbFail,
												  conduitApi.social.socialServicesEnum.Facebook,
												  options);
			}
			else
			{
				Utils.Helpers.funcWrapper(cbFail)('Social not supported');
			}
		}

		if (_userAccessToken)
		{
			Utils.Helpers.funcWrapper(cbSuccess)(_userAccessToken);
		}
		else if (options.fallbackToDefault)
		{
			if (_appAccessToken)
			{
				Utils.Helpers.funcWrapper(cbSuccess)(_appAccessToken);
			}
			else
			{
				getAccessTokenUsingLogin();
			}

		}
		else if (options.useLoginForm)
		{
			getAccessTokenUsingLogin();
		}
		else // _userAccessToken = "", and we will return it:
		{
			Utils.Helpers.funcWrapper(cbSuccess)(_userAccessToken);
		}

	};

	/**
	* get the _userId var.
	* NOTE: (according to the "option") If we don't have the _userId,
	* 	we will "login" to get it, and return it.
	*
	* @author Matanya
    
	me.getUserId = function (cbSuccess, cbFail, options)
	{
	var options = options || {};
	if (_userId || options.noLogin) 
	{
	Utils.Helpers.funcWrapper(cbSuccess)(_userId);
	}
	else		
	{
	if (conduitApi.social.isSupported()) 
	{
	var options = {useLoginForm : false};
	var cbGetAccessSuccess = function (tokens) 
	{
	// Update local vars:
	_userAccessToken = tokens.accessToken;
	_userId = tokens.fbId;
					
	// Return the wanted var:
	Utils.Helpers.funcWrapper(cbSuccess)(_userId);
					
	};
	conduitApi.social.getAccessTokens(cbGetAccessSuccess, 
	cbFail, 
	conduitApi.social.socialServicesEnum.Facebook,
	options);
	} 
	else 
	{
	Utils.Helpers.funcWrapper(cbFail)('Social not supported');
	}
	}
	};*/

	/**
	* Load Facebook library if needed
	*
	* @author Moshe Darzi
	*/
	var _loadFacebook = function (cbSuccess, cbFail)
	{
		var FACEBOOK_LOAD_TIMEOUT = 10000;
		// Already loaded
		if (typeof FB !== 'undefined')
		{
			cbSuccess();
		}
		else
		{
			// Callback from Facebook loading process.
			// It is called when the Facebook API is ready to work
			window.fbAsyncInit = function ()
			{
				if (!facebookFailed)
				{
					clearTimeout(facebookFailureTimeout);
					Utils.Helpers.funcWrapper(cbSuccess)();
				}
			}

			// Load Facebook library dynamically
			var e = document.createElement('script');
			e.type = 'text/javascript';
			e.src = 'http://connect.facebook.net/en_US/all.js';
			e.async = true;
			document.getElementById('fb-root').appendChild(e);

			var facebookFailed = false;

			// In case there is an error with facebook
			var facebookFailureTimeout = setTimeout(function ()
			{
				// Indicate failure in case of timeout too big
				facebookFailed = true;
				cbFail();
			}, FACEBOOK_LOAD_TIMEOUT);
		}
	};

	me.init = function (options)
	{
		// TODO: add an option to not do it twice (e.g.: "alreadyInit" flag)

		// TODO: we need different initOptions for native.
		if (PLATFORM === platformEnum.nativeApp)
		{
			// TBD... e.g.:

			var initOptions =
			{
				status: true, // check login status
				cookie: true, // enable cookies to allow the server to access the session
				xfbml: true, // parse XFBML
				oauth: true // enables OAuth 2.0
			};
			if (_appId)
			{
				initOptions.appId = _appId;
			}

			// Update options for native where Cordova plugin exists
			if (window.CDV && window.CDV.FB)
			{
				initOptions.nativeInterface = CDV.FB;
				initOptions.useCachedDialogs = false;
			}

			// Initialize the FB object, create the "LIKE" button we have just created
			FB.init(initOptions);
		}
		else
		{
			var loadFacebookSuccess = function ()
			{
				var initOptions =
				{
					status: true, // check login status
					cookie: true, // enable cookies to allow the server to access the session
					xfbml: true, // parse XFBML
					oauth: true // enables OAuth 2.0
				};
				if (_appId)
				{
					initOptions.appId = _appId;
				}

				// Initialize the FB object, create the "LIKE" button we have just created
				FB.init(initOptions);
			};

			var loadFacebookFail = function ()
			{
				// What to do?
			};

			_loadFacebook(loadFacebookSuccess, loadFacebookFail);
		}
	};

	/**
	* This function adds a "like" button to html container element.
	* this function is called after we have the "link" for the 
	* "like", so we can show the "like" dialog now
	* 
	* @param link - The link of the fun page to "like"
	* @param $container - The container that we will add the "like" into it.
	* @author Matanya
	*/
	me.addLikeButton = function (link, $container, options)
	{

		options = options || {};

		// TODO: We can change the language! e.g.: fr_FR
		var params =
        {
        	link: link,
        	width: 200,
        	// TODO: we can change the light/dark property as well.
        	colorscheme: 'light'
        }

		// Add the "like stuff":
		$container.html(faceBookTemplates.likeButton(params));

		me.init();

		if (options.cbClick)
		{
			//"bind event" on click on the like button:
			FB.Event.subscribe('edge.create',
  		    function (response)
  		    {
  		    	options.cbClick(response);
  		    });
		}
	};

	me.isLike = function (pageId, cbSuccess, cbFail, options)
	{
		var cbGetAccessTokenSuccess = function (accessToken)
		{
			// Check if the user already liked the page
			FB.api(
			{
				method: 'pages.isFan',
				page_id: pageId,
				uid: _userId, // NOTE: Now we sure we have it.
				access_token: accessToken
			}, function (response)
			{
				if (typeof (response) === "boolean")
				{
					Utils.Helpers.funcWrapper(cbSuccess)(response);
				}
				else
				{
					Utils.Helpers.funcWrapper(cbFail)(response);
				}
			});
		}

		getAccessToken(cbGetAccessTokenSuccess, cbFail, { useLoginForm: true });
	};

	me.getPageData = function (pageId, cbSuccess, cbFail, options)
	{

		var cbGetAccessTokenSuccess = function (accessToken)
		{
			// TODO: get only the wanted fields!
			options = options || {};

			FB.api('/' + pageId,
			{
				limit: options.limit,
				access_token: accessToken
			}, function (response)
			{
				if (!response || response.error)
				{
					Utils.Helpers.funcWrapper(cbFail)(response);
				}
				else
				{
					Utils.Helpers.funcWrapper(cbSuccess)(response);
				}
			});
		}
		getAccessToken(cbGetAccessTokenSuccess, cbFail, { fallbackToDefault: true });
	};

	me.getPosts = function (pageId, cbSuccess, cbFail, options, postsSource)
	{
		var cbGetAccessTokenSuccess = function (accessToken)
		{
			// TODO: get only the wanted fields!
			options = options || {};
			options.limit = options.limit || POSTS_TAKE_COUNT;

			postsSource = postsSource || "feed";

			//use the AT that we have:
			options.access_token = accessToken;

			FB.api('/' + pageId + '/' + postsSource, options, function (response)
			{
				if (!response || response.error)
				{
					Utils.Helpers.funcWrapper(cbFail)(response);
				}
				else
				{
					Utils.Helpers.funcWrapper(cbSuccess)(response);
				}
			});
		}
		getAccessToken(cbGetAccessTokenSuccess, cbFail, { fallbackToDefault: true });
	};

	me.getComments = function (objectId, cbSuccess, cbFail, options)
	{
		var cbGetAccessTokenSuccess = function (accessToken)
		{
			// TODO: get only the wanted fields!
			options = options || {};
			options.limit = options.limit || COMMENTS_TAKE_COUNT;

			//use the AT that we have:
			options.access_token = accessToken;

			FB.api('/' + objectId + '/comments', options, function (response)
			{
				if (!response || response.error)
				{
					Utils.Helpers.funcWrapper(cbFail)(response);
				}
				else
				{
					Utils.Helpers.funcWrapper(cbSuccess)(response);
				}
			});
		}
		getAccessToken(cbGetAccessTokenSuccess, cbFail, { fallbackToDefault: true });
	};

	me.getItem = function (objectId, cbSuccess, cbFail, options)
	{
		var cbGetAccessTokenSuccess = function (accessToken)
		{
			// TODO: get only the wanted fields!
			options = options || {};

			//use the AT that we have:
			options.access_token = accessToken;

			FB.api('/' + objectId, options, function (response)
			{
				if (!response || response.error)
				{
					Utils.Helpers.funcWrapper(cbFail)(response);
				}
				else
				{
					Utils.Helpers.funcWrapper(cbSuccess)(response);
				}
			});
		}
		getAccessToken(cbGetAccessTokenSuccess, cbFail, { fallbackToDefault: true });
	};

	me.likeItem = function (objectId, cbSuccess, cbFail, options)
	{
		var cbGetAccessTokenSuccess = function (accessToken)
		{
			options = options || {};

			FB.api('/' + objectId + '/likes', 'post',
			{
				access_token: accessToken
			}, function (response)
			{
				if (!response || response.error)
				{
					Utils.Helpers.funcWrapper(cbFail)(response);
				}
				else
				{
					Utils.Helpers.funcWrapper(cbSuccess)(response);
				}
			});
		}
		getAccessToken(cbGetAccessTokenSuccess, cbFail, { useLoginForm: true });
	};

	me.unlikeItem = function (objectId, cbSuccess, cbFail, options)
	{
		var cbGetAccessTokenSuccess = function (accessToken)
		{
			options = options || {};

			FB.api('/' + objectId + '/likes', 'delete',
			{
				access_token: accessToken
			}, function (response)
			{
				if (!response || response.error)
				{
					Utils.Helpers.funcWrapper(cbFail)(response);
				}
				else
				{
					Utils.Helpers.funcWrapper(cbSuccess)(response);
				}
			});
		}
		getAccessToken(cbGetAccessTokenSuccess, cbFail, { useLoginForm: true });
	};

	me.commentItem = function (objectId, message, cbSuccess, cbFail, options)
	{
		var cbGetAccessTokenSuccess = function (accessToken)
		{
			options = options || {};

			FB.api('/' + objectId + '/comments', 'post',
			{
				message: message,
				access_token: accessToken
			}, function (response)
			{

				if (!response || response.error)
				{
					Utils.Helpers.funcWrapper(cbFail)(response);
				}
				else
				{
					Utils.Helpers.funcWrapper(cbSuccess)(response);
				}
			});
		}
		getAccessToken(cbGetAccessTokenSuccess, cbFail, { useLoginForm: true });
	};

	me.postMessage = function (pageId, message, cbSuccess, cbFail, options)
	{
		var cbGetAccessTokenSuccess = function (accessToken)
		{
			options = options || {};

			FB.api('/' + pageId + '/feed', 'post',
			{
				message: message,
				access_token: accessToken
			}, function (response)
			{

				if (!response || response.error)
				{
					Utils.Helpers.funcWrapper(cbFail)(response);
				}
				else
				{
					Utils.Helpers.funcWrapper(cbSuccess)(response);
				}
			});
		}
		getAccessToken(cbGetAccessTokenSuccess, cbFail, { useLoginForm: true });
	};

	me.getUserData = function (cbSuccess, cbFail)
	{
		me.init();

		function cbGetAccessTokenSuccess(tokens)
		{
			function cbGetUserData(response)
			{
				Utils.Helpers.funcWrapper(cbSuccess)(response);
			}
			FB.api('/' + _userId, cbGetUserData);
		}
		function cbGetAccessTokenFail()
		{
			Utils.Helpers.funcWrapper(cbSuccess)(cbFail);
		}
		getAccessToken(cbGetAccessTokenSuccess, cbGetAccessTokenFail, { useLoginForm: true });
	};

	me.setAppAccessToken = function (appAccessToken)
	{
		_appAccessToken = appAccessToken;
	};

	return me;
} ());
